@@ -24,6 +24,7 @@
 #include "../src/output.h"
 #include "../src/mode.h"
 #include "../src/getconfigoperation.h"
+#include "../src/setconfigoperation.h"
 #include "../src/backendmanager_p.h"
 
 using namespace KScreen;
@@ -43,7 +44,9 @@ private Q_SLOTS:
     void clonesOutput();
     void configCanBeApplied();
     void supportedFeatures();
+    void testInvalidMode();
     void cleanupTestCase();
+    void testOutputPositionNormalization();
 };
 
 ConfigPtr testScreenConfig::getConfig()
@@ -64,6 +67,7 @@ ConfigPtr testScreenConfig::getConfig()
 
 void testScreenConfig::initTestCase()
 {
+    qputenv("KSCREEN_LOGGING", "false");
     qputenv("KSCREEN_BACKEND", "Fake");
 }
 
@@ -105,6 +109,7 @@ void testScreenConfig::singleOutput()
     QCOMPARE(output->currentModeId(), QLatin1String("3"));
     QCOMPARE(output->preferredModeId(), QLatin1String("3"));
     QCOMPARE(output->rotation(), Output::None);
+    QCOMPARE(output->scale(), 1.0);
     QCOMPARE(output->isConnected(), true);
     QCOMPARE(output->isEnabled(), true);
     QCOMPARE(output->isPrimary(), true);
@@ -155,6 +160,7 @@ void testScreenConfig::multiOutput()
     QCOMPARE(output->currentModeId(), QLatin1String("4"));
     QCOMPARE(output->preferredModeId(), QLatin1String("4"));
     QCOMPARE(output->rotation(), Output::None);
+    QCOMPARE(output->scale(), 1.4);
     QCOMPARE(output->isConnected(), true);
     QCOMPARE(output->isEnabled(), true);
     QCOMPARE(output->isPrimary(), false);
@@ -238,13 +244,83 @@ void testScreenConfig::supportedFeatures()
     ConfigPtr config = getConfig();
 
     QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::None));
+    QVERIFY(!config->supportedFeatures().testFlag(KScreen::Config::Feature::Writable));
+    QVERIFY(!config->supportedFeatures().testFlag(KScreen::Config::Feature::PrimaryDisplay));
+    QVERIFY(!config->supportedFeatures().testFlag(KScreen::Config::Feature::PerOutputScaling));
 
     config->setSupportedFeatures(KScreen::Config::Feature::Writable | KScreen::Config::Feature::PrimaryDisplay);
-
     QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::Writable));
     QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::PrimaryDisplay));
+
+    config->setSupportedFeatures(KScreen::Config::Feature::None);
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::None));
+
+    config->setSupportedFeatures(KScreen::Config::Feature::PerOutputScaling | KScreen::Config::Feature::Writable);
+    QVERIFY(!config->supportedFeatures().testFlag(KScreen::Config::Feature::None));
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::Writable));
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::PerOutputScaling));
+
+    config->setSupportedFeatures(KScreen::Config::Feature::PerOutputScaling | KScreen::Config::Feature::Writable | KScreen::Config::Feature::PrimaryDisplay);
+    QVERIFY(!config->supportedFeatures().testFlag(KScreen::Config::Feature::None));
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::Writable));
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::PrimaryDisplay));
+    QVERIFY(config->supportedFeatures().testFlag(KScreen::Config::Feature::PerOutputScaling));
 }
 
+void testScreenConfig::testInvalidMode()
+{
+    ModeList modes;
+    ModePtr invalidMode = modes.value("99");
+    QVERIFY(invalidMode.isNull());
+
+    auto output = new KScreen::Output();
+    auto currentMode = output->currentMode();
+    QVERIFY(currentMode.isNull());
+    QVERIFY(!currentMode);
+    delete output;
+}
+
+void testScreenConfig::testOutputPositionNormalization()
+{
+    qputenv("KSCREEN_BACKEND_ARGS", "TEST_DATA=" TEST_DATA "multipleoutput.json");
+
+    const ConfigPtr config = getConfig();
+    QVERIFY(!config.isNull());
+    auto left = config->outputs().first();
+    auto right = config->outputs().last();
+    QVERIFY(!left.isNull());
+    QVERIFY(!right.isNull());
+    left->setPos(QPoint(-5000, 700));
+    right->setPos(QPoint(-3720, 666));
+    QCOMPARE(left->pos(), QPoint(-5000, 700));
+    QCOMPARE(right->pos(), QPoint(-3720, 666));
+
+    // start a set operation to fix up the positions
+    {
+        auto setop = new SetConfigOperation(config);
+        setop->exec();
+    }
+    QCOMPARE(left->pos(), QPoint(0, 34));
+    QCOMPARE(right->pos(), QPoint(1280, 0));
+
+    // make sure it doesn't touch a valid config
+    {
+        auto setop = new SetConfigOperation(config);
+        setop->exec();
+    }
+    QCOMPARE(left->pos(), QPoint(0, 34));
+    QCOMPARE(right->pos(), QPoint(1280, 0));
+
+    // positions of single outputs should be at 0, 0
+    left->setEnabled(false);
+    {
+        auto setop = new SetConfigOperation(config);
+        setop->exec();
+    }
+    QCOMPARE(right->pos(), QPoint());
+}
+
+
 QTEST_MAIN(testScreenConfig)
 
 #include "testscreenconfig.moc"
